(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = global || self, factory(global.ReactIntersectionObserver = {}, global.react));
}(this, (function (exports, React) {
  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    subClass.__proto__ = superClass;
  }

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  var ObserverMap = new Map();
  var RootIds = new Map();
  var consecutiveRootId = 0;
  /**
   * Generate a unique ID for the root element
   * @param root
   */

  function getRootId(root) {
    if (!root) return '';
    if (RootIds.has(root)) return RootIds.get(root);
    consecutiveRootId += 1;
    RootIds.set(root, consecutiveRootId.toString());
    return RootIds.get(root);
  }
  /**
   * Convert the options to a string Id, based on the values.
   * Ensures we can reuse the same observer for, when observer elements with the same options.
   * @param options
   */


  function optionsToId(options) {
    return Object.keys(options).sort().map(function (key) {
      return key + "_" + (key === 'root' ? getRootId(options.root) : options[key]);
    }).join('|');
  }

  function createObserver(options) {
    // Create a unique ID for this observer instance, based on the root, root margin and threshold.
    var id = optionsToId(options);
    var instance = ObserverMap.get(id);

    if (!instance) {
      // Create a map of elements this observer is going to observe. Each element has a list of callbacks that should be triggered, once it comes into view.
      var elements = new Map();
      var observer = new IntersectionObserver(function (entries) {
        entries.forEach(function (entry) {
          var _elements$get;

          // While it would be nice if you could just look at isIntersecting to determine if the component is inside the viewport, browsers can't agree on how to use it.
          // -Firefox ignores `threshold` when considering `isIntersecting`, so it will never be false again if `threshold` is > 0
          var inView = entry.isIntersecting && observer.thresholds.some(function (threshold) {
            return entry.intersectionRatio >= threshold;
          }); // @ts-ignore support IntersectionObserver v2

          if (options.trackVisibility && typeof entry.isVisible === 'undefined') {
            // The browser doesn't support Intersection Observer v2, falling back to v1 behavior.
            // @ts-ignore
            entry.isVisible = inView;
          }

          (_elements$get = elements.get(entry.target)) == null ? void 0 : _elements$get.forEach(function (callback) {
            callback(inView, entry);
          });
        });
      }, options);
      instance = {
        id: id,
        observer: observer,
        elements: elements
      };
      ObserverMap.set(id, instance);
    }

    return instance;
  }

  function observe(element, callback, options) {
    if (options === void 0) {
      options = {};
    }

    if (!element) return function () {}; // An observer with the same options can be reused, so lets use this fact

    var _createObserver = createObserver(options),
        id = _createObserver.id,
        observer = _createObserver.observer,
        elements = _createObserver.elements; // Register the callback listener for this element


    var callbacks = elements.get(element) || [];

    if (!elements.has(element)) {
      elements.set(element, callbacks);
    }

    callbacks.push(callback);
    observer.observe(element);
    return function unobserve() {
      // Remove the callback from the callback list
      callbacks.splice(callbacks.indexOf(callback), 1);

      if (callbacks.length === 0) {
        // No more callback exists for element, so destroy it
        elements["delete"](element);
        observer.unobserve(element);
      }

      if (elements.size === 0) {
        // No more elements are being observer by this instance, so destroy it
        observer.disconnect();
        ObserverMap["delete"](id);
      }
    };
  }

  function isPlainChildren(props) {
    return typeof props.children !== 'function';
  }
  /**
   * Monitors scroll, and triggers the children function with updated props
   */


  var InView = /*#__PURE__*/function (_React$Component) {
    _inheritsLoose(InView, _React$Component);

    function InView() {
      var _this;

      _this = _React$Component.apply(this, arguments) || this;
      _this.state = {
        inView: false,
        entry: undefined
      };
      _this.node = null;
      _this._unobserveCb = null;

      _this.handleNode = function (node) {
        if (_this.node) {
          _this.unobserve();

          if (!node && !_this.props.triggerOnce && !_this.props.skip) {
            _this.setState({
              inView: false,
              entry: undefined
            });
          }
        }

        _this.node = node ? node : null;

        _this.observeNode();
      };

      _this.handleChange = function (inView, entry) {
        // Only trigger a state update if inView has changed.
        // This prevents an unnecessary extra state update during mount, when the element stats outside the viewport
        if (inView !== _this.state.inView || inView) {
          _this.setState({
            inView: inView,
            entry: entry
          });
        }

        if (_this.props.onChange) {
          // If the user is actively listening for onChange, always trigger it
          _this.props.onChange(inView, entry);
        }
      };

      return _this;
    }

    var _proto = InView.prototype;

    _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
      // If a IntersectionObserver option changed, reinit the observer
      if (prevProps.rootMargin !== this.props.rootMargin || prevProps.root !== this.props.root || prevProps.threshold !== this.props.threshold || prevProps.skip !== this.props.skip || prevProps.trackVisibility !== this.props.trackVisibility || prevProps.delay !== this.props.delay) {
        this.unobserve();
        this.observeNode();
      }

      if (prevState.inView !== this.state.inView) {
        if (this.state.inView && this.props.triggerOnce) {
          this.unobserve();
          this.node = null;
        }
      }
    };

    _proto.componentWillUnmount = function componentWillUnmount() {
      if (this.node) {
        this.unobserve();
        this.node = null;
      }
    };

    _proto.observeNode = function observeNode() {
      if (!this.node || this.props.skip) return;
      var _this$props = this.props,
          threshold = _this$props.threshold,
          root = _this$props.root,
          rootMargin = _this$props.rootMargin,
          trackVisibility = _this$props.trackVisibility,
          delay = _this$props.delay;
      this._unobserveCb = observe(this.node, this.handleChange, {
        threshold: threshold,
        root: root,
        rootMargin: rootMargin,
        // @ts-ignore
        trackVisibility: trackVisibility,
        // @ts-ignore
        delay: delay
      });
    };

    _proto.unobserve = function unobserve() {
      if (this._unobserveCb) {
        this._unobserveCb();

        this._unobserveCb = null;
      }
    };

    _proto.render = function render() {
      var _this$state = this.state,
          inView = _this$state.inView,
          entry = _this$state.entry;

      if (!isPlainChildren(this.props)) {
        return this.props.children({
          inView: inView,
          entry: entry,
          ref: this.handleNode
        });
      }

      var _this$props2 = this.props,
          children = _this$props2.children,
          as = _this$props2.as,
          tag = _this$props2.tag,
          props = _objectWithoutPropertiesLoose(_this$props2, ["children", "as", "tag", "triggerOnce", "threshold", "root", "rootMargin", "onChange", "skip", "trackVisibility", "delay"]);

      return /*#__PURE__*/React.createElement(as || tag || 'div', _extends({
        ref: this.handleNode
      }, props), children);
    };

    return InView;
  }(React.Component);
  InView.displayName = 'InView';
  InView.defaultProps = {
    threshold: 0,
    triggerOnce: false
  };

  /* eslint-disable react-hooks/exhaustive-deps */
  var initialState = {
    inView: false,
    entry: undefined
  };
  function useInView(options) {
    if (options === void 0) {
      options = {};
    }

    var unobserve = React.useRef();

    var _React$useState = React.useState(initialState),
        state = _React$useState[0],
        setState = _React$useState[1];

    var setRef = React.useCallback(function (node) {
      if (unobserve.current !== undefined) {
        unobserve.current();
        unobserve.current = undefined;
      }

      if (options.skip) {
        return;
      }

      if (node) {
        unobserve.current = observe(node, function (inView, entry) {
          setState({
            inView: inView,
            entry: entry
          });

          if (entry.isIntersecting && options.triggerOnce && unobserve.current) {
            // If it should only trigger once, unobserve the element after it's inView
            unobserve.current();
            unobserve.current = undefined;
          }
        }, options);
      }
    }, [options.threshold, options.root, options.rootMargin, options.triggerOnce, options.skip, options.trackVisibility, options.delay]);
    React.useEffect(function () {
      if (!unobserve.current && !options.triggerOnce && !options.skip) {
        // If we don't have a ref, then reset the state (unless the hook is set to only `triggerOnce` or `skip`)
        // This ensures we correctly reflect the current state - If you aren't observing anything, then nothing is inView
        setState(initialState);
      }
    });
    var result = [setRef, state.inView, state.entry]; // Support object destructuring, by adding the specific values.

    result.ref = result[0];
    result.inView = result[1];
    result.entry = result[2];
    return result;
  }

  exports.InView = InView;
  exports.default = InView;
  exports.useInView = useInView;

})));
//# sourceMappingURL=react-intersection-observer.umd.js.map
